"use client";

import { zodResolver } from "@hookform/resolvers/zod";
import { useForm } from "react-hook-form";
import { z } from "zod";

import { Button } from "@/components/ui/button";
import {
  Form,
  FormControl,
  FormField,
  FormItem,
  FormLabel,
  FormMessage,
} from "@/components/ui/form";
import { toast } from "@/components/ui/use-toast";
import { Input } from "./ui/input";
import { Popover, PopoverContent, PopoverTrigger } from "./ui/popover";
import { CaretSortIcon, CheckIcon } from "@radix-ui/react-icons";
import {
  Command,
  CommandEmpty,
  CommandGroup,
  CommandInput,
  CommandItem,
} from "./ui/command";
import { cn } from "@/lib/utils";
import { CommandList } from "cmdk";
import { useState } from "react";
import { Evaluation } from "@/types/evaluation";

const statuses = [
  "uploading",
  "new",
  "requested",
  "extraction_running",
  "skills_extracted",
  "evaluation_running",
  "score_requested",
  "course_creation",
  "finished",
  "deleted",
  "extraction_failed",
  "evaluation_failed",
] as const;

const uploaders = ["johndoe@gmail.com", "danabramov@gmail.com"] as const;
const developers = ["danabramov@gmail.com", "johndoe@gmail.com"] as const;

const displayFormSchema = z.object({
  status: z.string().trim().min(1, "Status is required"),
  uploadTime: z.string().refine((value) => /^\d{4}-\d{2}-\d{2}$/.test(value), {
    message: "Upload time should be in the format YYYY-MM-DD",
  }),
  uploaderAccount: z
    .string()
    .trim()
    .email()
    .min(1, "Please select a uploader account"),
  developerUser: z
    .string()
    .trim()
    .email()
    .min(1, "Please select a developer user"),
});

type DisplayFormValues = z.infer<typeof displayFormSchema>;

// const defaultValues: Partial<DisplayFormValues> = {
//   status: "",
//   uploadTime: "",
//   uploaderAccount: "",
//   developerUser: "",
// };

type Props = {
  evaluation?: Evaluation;
  onSubmit: () => void;
  btn1_content: string;
  btn2_content: string;
};

export default function EvaluationForm({
  evaluation,
  onSubmit: onFormSubmit,
  btn1_content,
  btn2_content,
}: Props) {
  const [isStatusPopoverOpen, setIsStatusPopoverOpen] = useState(false);
  const [isUploaderPopoverOpen, setIsUploaderPopoverOpen] = useState(false);
  const [isDeveloperPopoverOpen, setIsDeveloperPopoverOpen] = useState(false);

  const form = useForm<DisplayFormValues>({
    resolver: zodResolver(displayFormSchema),
    defaultValues: {
      status: evaluation?.status || "",
      uploadTime: evaluation?.uploadTime || "",
      uploaderAccount: evaluation?.uploaderAccount || "",
      developerUser: evaluation?.developerUser || "",
    },
  });

  function onSubmit(data: DisplayFormValues) {
    toast({
      title: "You submitted the following values:",
      description: (
        <pre className="mt-2 w-[340px] rounded-md bg-slate-950 p-4">
          <code className="text-white">{JSON.stringify(data, null, 2)}</code>
        </pre>
      ),
    });

    onFormSubmit();
  }

  return (
    <Form {...form}>
      <form onSubmit={form.handleSubmit(onSubmit)} className="space-y-8">
        <FormField
          control={form.control}
          name="status"
          render={({ field }) => (
            <FormItem className="flex flex-col">
              <FormLabel>Status</FormLabel>
              <Popover
                open={isStatusPopoverOpen}
                onOpenChange={setIsStatusPopoverOpen}
              >
                <PopoverTrigger asChild>
                  <FormControl>
                    <Button
                      variant="outline"
                      role="combobox"
                      className={cn(
                        "w-full justify-between",
                        !field.value && "text-muted-foreground"
                      )}
                    >
                      {field.value
                        ? statuses.find((status) => status === field.value)
                        : "Select Status"}
                      <CaretSortIcon className="ml-2 h-4 w-4 shrink-0 opacity-50" />
                    </Button>
                  </FormControl>
                </PopoverTrigger>
                <PopoverContent className="w-[300px]  md:w-[650px] p-0">
                  <Command>
                    <CommandList>
                      <CommandInput
                        placeholder="Search status..."
                        className="h-9"
                      />
                      <CommandEmpty>No status found.</CommandEmpty>
                      <CommandGroup>
                        {statuses.map((status) => (
                          <CommandItem
                            value={status}
                            key={status}
                            onSelect={() => {
                              form.setValue("status", status);
                              setIsStatusPopoverOpen(false);
                            }}
                          >
                            {status}
                            <CheckIcon
                              className={cn(
                                "ml-auto h-4 w-4",
                                status === field.value
                                  ? "opacity-100"
                                  : "opacity-0"
                              )}
                            />
                          </CommandItem>
                        ))}
                      </CommandGroup>
                    </CommandList>
                  </Command>
                </PopoverContent>
              </Popover>
              <FormMessage />
            </FormItem>
          )}
        />

        <FormField
          control={form.control}
          name={`uploadTime`}
          render={({ field }) => (
            <FormItem>
              <FormLabel>Upload Time</FormLabel>
              <FormControl>
                <Input type="date" {...field} />
              </FormControl>
              <FormMessage />
            </FormItem>
          )}
        />

        <FormField
          control={form.control}
          name="uploaderAccount"
          render={({ field }) => (
            <FormItem className="flex flex-col">
              <FormLabel>Uploader Account</FormLabel>
              <Popover
                open={isUploaderPopoverOpen}
                onOpenChange={setIsUploaderPopoverOpen}
              >
                <PopoverTrigger asChild>
                  <FormControl>
                    <Button
                      variant="outline"
                      role="combobox"
                      className={cn(
                        "w-full justify-between",
                        !field.value && "text-muted-foreground"
                      )}
                    >
                      {field.value
                        ? uploaders.find((uploader) => uploader === field.value)
                        : "Select uploader"}
                      <CaretSortIcon className="ml-2 h-4 w-4 shrink-0 opacity-50" />
                    </Button>
                  </FormControl>
                </PopoverTrigger>
                <PopoverContent className="w-[300px]  md:w-[650px] p-0">
                  <Command>
                    <CommandList>
                      <CommandInput
                        placeholder="Search uploader..."
                        className="h-9"
                      />
                      <CommandEmpty>No uploader found.</CommandEmpty>
                      <CommandGroup>
                        {uploaders.map((uploader) => (
                          <CommandItem
                            value={uploader}
                            key={uploader}
                            onSelect={() => {
                              form.setValue("uploaderAccount", uploader);
                              setIsUploaderPopoverOpen(false);
                            }}
                          >
                            {uploader}
                            <CheckIcon
                              className={cn(
                                "ml-auto h-4 w-4",
                                uploader === field.value
                                  ? "opacity-100"
                                  : "opacity-0"
                              )}
                            />
                          </CommandItem>
                        ))}
                      </CommandGroup>
                    </CommandList>
                  </Command>
                </PopoverContent>
              </Popover>
              <FormMessage />
            </FormItem>
          )}
        />
        <FormField
          control={form.control}
          name="developerUser"
          render={({ field }) => (
            <FormItem className="flex flex-col">
              <FormLabel>Developer User</FormLabel>
              <Popover
                open={isDeveloperPopoverOpen}
                onOpenChange={setIsDeveloperPopoverOpen}
              >
                <PopoverTrigger asChild>
                  <FormControl>
                    <Button
                      variant="outline"
                      role="combobox"
                      className={cn(
                        "w-full justify-between",
                        !field.value && "text-muted-foreground"
                      )}
                    >
                      {field.value
                        ? developers.find(
                            (developer) => developer === field.value
                          )
                        : "Select developer"}
                      <CaretSortIcon className="ml-2 h-4 w-4 shrink-0 opacity-50" />
                    </Button>
                  </FormControl>
                </PopoverTrigger>
                <PopoverContent className="w-[300px]  md:w-[650px] p-0">
                  <Command>
                    <CommandList>
                      <CommandInput
                        placeholder="Search developer..."
                        className="h-9"
                      />
                      <CommandEmpty>No developer found.</CommandEmpty>
                      <CommandGroup>
                        {developers.map((developer) => (
                          <CommandItem
                            value={developer}
                            key={developer}
                            onSelect={() => {
                              form.setValue("developerUser", developer);
                              setIsDeveloperPopoverOpen(false);
                            }}
                          >
                            {developer}
                            <CheckIcon
                              className={cn(
                                "ml-auto h-4 w-4",
                                developer === field.value
                                  ? "opacity-100"
                                  : "opacity-0"
                              )}
                            />
                          </CommandItem>
                        ))}
                      </CommandGroup>
                    </CommandList>
                  </Command>
                </PopoverContent>
              </Popover>
              <FormMessage />
            </FormItem>
          )}
        />

        <div className="flex items-center gap-4 flex-col sm:flex-row">
          <Button className="w-full" size="lg">
            {btn1_content}
          </Button>
          <Button variant="secondary" className="w-full" size="lg">
            {btn2_content}
          </Button>
        </div>
      </form>
    </Form>
  );
}
